from Products.CMFCore.utils import getToolByName
from DateTime import DateTime
from zope.component import queryUtility
from interfaces import IStateTranslationUtility
import logging

def default_state_translation(source_state, target_workflow_id):
    return source_state

def setWorkflowState(obj, event):
    """
    """
    info = logging.getLogger("setWorkflowState").info
    
    wtool = getToolByName(obj, 'portal_workflow')
    review_state = wtool.getInfoFor(event.object, "review_state") 
    # try to get a translation table
    
    info('Workflow used %s' % event.data["workflow_id"])
    
    state_translation_utility = queryUtility(IStateTranslationUtility, name=event.data["workflow_id"])
    if not state_translation_utility:
        info('using default_state_translation')
        state_translation_utility = default_state_translation
    
    
    
    wtool = getToolByName(obj, 'portal_workflow')
    chain = wtool.getChainFor(obj)

    info(chain)

    for workflow_id in chain:
        workflow = wtool.getWorkflowById(workflow_id)
        if workflow:
            target_state = state_translation_utility(event.data["review_state"], workflow_id)
            info(target_state)
            if not target_state:
                target_state = workflow.initial_state
            target = {
                 'action'       : None,
                 'actor'        : 'gsxml',
                 'comments'     : 'imported',
                 'review_state' : target_state,
                 'time'         : DateTime(),
               }

            wtool.setStatusOf(workflow_id, obj, target)
            workflow.updateRoleMappingsFor(obj)


#wtool.doActionFor(obj, 'protect')



from zope.lifecycleevent.interfaces import IObjectModifiedEvent
from zope.interface import Interface, Attribute


class IObjectReviewStateDeserializedEvent(IObjectModifiedEvent):
    """
    """
    
    data = Attribute("Resolved data from namespace.")
    
class IStateTranslationUtility(Interface):
    """
    """


from DateTime import DateTime
from Products.CMFCore.utils import getToolByName
from Products.Marshall.handlers.atxml import XmlNamespace
from Products.Marshall.handlers.atxml import SchemaAttribute
from Products.Marshall import utils
from zope import event
from collective.marshall.workflow.events import ObjectReviewStateDeserializedEvent

StateRNGSchemaFragment = '''
  <define name="StateInfo"
          xmlns:cmf="http://www.brainson.de/namespaces/workflow"
          datatypeLibrary="http://www.w3.org/2001/XMLSchema-datatypes"
          xmlns="http://relaxng.org/ns/structure/1.0">
           <element name="workflow:status">
             <attribute name="id" /> 
             <text /> 
           </element>
          </zeroOrMore>
  </define>
'''

class StateAttribute(SchemaAttribute):

    def get(self, instance):
        return instance.getPortalTypeName()
    
    def processXmlValue(self, context, value):
        """ callback to process text nodes
        """
        value = value and value.strip()
        if not value:
            return
        data = context.getDataFor( self.namespace.xmlns )
        data[self.name] = {"review_state": value,
                           "workflow_id" : context.node.get("id"),
                          }
 

    def deserialize(self, instance, ns_data):
        data = ns_data.get(self.name)
        event.notify(ObjectReviewStateDeserializedEvent(instance, data))
        #instance._setPortalTypeName( value )

    def serialize(self, dom, parent_node, instance):
        wf_tool = getToolByName(instance, 'portal_workflow')
        value = wf_tool.getInfoFor(instance, "review_state")
        elname = "%s:%s"%(self.namespace.prefix, self.name)
        node = dom.createElementNS( Workflow.xmlns, elname )
        
        workflow_id = dom.createAttribute("id")
        #state = dom.createAttribute("state")
        
        workflow_id.value = ",".join(wf_tool.getChainFor(instance))
        #state.value = value
        
        node.setAttributeNode(workflow_id)
        #node.setAttributeNode(state)
        
        value_node = dom.createTextNode( str( value ) )
        node.appendChild( value_node )
        node.normalize()
        parent_node.appendChild( node )

class Workflow(XmlNamespace):
    
    xmlns = 'http://www.brainson.de/namespaces/workflow/'
    prefix = 'workflow'
    attributes = (
        StateAttribute('status'),
        )

    def getSchemaInfo( self ):
        return [
            ( "StateInfo", "optional", StateRNGSchemaFragment ),
              ]
      



from zope.interface import implements
from zope.lifecycleevent import ObjectModifiedEvent
from interfaces import IObjectReviewStateDeserializedEvent

class ObjectReviewStateDeserializedEvent(ObjectModifiedEvent):
    """
        The marshaller in namespaces.wfstate received a review_state.
        Let's transition the object into the desired state.
    """
    implements(IObjectReviewStateDeserializedEvent)
    
    def __init__(self, object, data):
        self.object = object
        self.data = data
 
